@@ -3,42 +3,30 @@
 #![deny(warnings)]
 #[cfg(test)]
 mod tests;
-use std::collections::HashMap;
+use std::collections::BTreeMap;
 #[cfg(test)]
 extern crate quickcheck;
 #[cfg(test)]
 #[macro_use(quickcheck)]
 extern crate quickcheck_macros;
 
-#[derive(Debug, PartialEq, Eq, Hash, Clone, Copy)]
+#[derive(Debug, PartialEq, Eq, Hash, Clone, Copy, PartialOrd, Ord)]
 enum Node {
     P((usize, usize)),
     Root,
 }
 
-struct EditPath {
-    nodes_map: HashMap<Node, Node>,
-    cur: Node,
-}
-
-impl<'a> Iterator for EditPath {
-    type Item = (usize, usize);
-    fn next(&mut self) -> Option<Self::Item> {
-        match self.cur {
-            Node::Root => None,
-            Node::P(cur) => {
-                self.cur = *self.nodes_map.get(&Node::P(cur)).unwrap();
-                Some(cur)
-            }
-        }
-    }
-}
-
-/// Returns an iterator over the shotest path of the edit graph based on Myers' diff algorithm.
+/// Returns an iterator over the shotest path of the edit graph based on Myers'
+/// diff algorithm.
 ///
 /// See [An O(ND) Difference Algorithm and Its Variations](http://www.xmailserver.org/diff2.pdf)
 #[allow(clippy::many_single_char_names)]
-fn get_shortest_edit_path_myers<A, B, F>(a: &[A], b: &[B], is_eq: F) -> EditPath
+fn get_shortest_edit_path<A, B, F>(
+    a: &[A],
+    b: &[B],
+    is_eq: F,
+    get_path: bool,
+) -> (usize, Option<impl Iterator<Item = (usize, usize)>>)
 where
     F: Fn(&A, &B) -> bool,
 {
@@ -47,7 +35,12 @@ where
     let bound = n + m;
     let get_y = |x, k| x + bound - k;
     let mut v = vec![0; 2 * bound + 1];
-    let mut nodes_map = HashMap::new();
+    let mut nodes_map = if get_path {
+        Some(BTreeMap::new())
+    } else {
+        None
+    };
+    let mut distance = !0;
     'outer: for d in 0..=bound {
         for k in ((bound - d)..=bound + d).step_by(2) {
             let (mut x, parent) = if d == 0 {
@@ -60,22 +53,38 @@ where
                 (px + 1, Node::P((px, get_y(px, k - 1))))
             };
             let mut y = get_y(x, k);
-            nodes_map.insert(Node::P((x, y)), parent);
+            if get_path {
+                nodes_map.as_mut().unwrap().insert(Node::P((x, y)), parent);
+            }
             while x < n && y < m && is_eq(&a[x], &b[y]) {
-                nodes_map.insert(Node::P((x + 1, y + 1)), Node::P((x, y)));
                 x += 1;
                 y += 1;
             }
             v[k] = x;
             if x >= n && y >= m {
+                distance = d;
                 break 'outer;
             }
         }
     }
-
-    EditPath {
-        nodes_map,
-        cur: Node::P((n, m)),
+    debug_assert_ne!(distance, !0);
+    if get_path {
+        let mut cur = Node::P((n, m));
+        let nodes_map = nodes_map.unwrap();
+        let path = std::iter::from_fn(move || match cur {
+            Node::Root => None,
+            Node::P(ncur) => {
+                cur = if let Some(cur) = nodes_map.get(&Node::P(ncur)) {
+                    *cur
+                } else {
+                    Node::P((ncur.0 - 1, ncur.1 - 1))
+                };
+                Some(ncur)
+            }
+        });
+        (distance, Some(path))
+    } else {
+        (distance, None)
     }
 }
 
@@ -99,6 +108,12 @@ pub type Diff = Vec<Option<usize>>;
 
 /// Returns the correspondence between two sequences.
 ///
+/// The return value is a pair of tuples. The first tuple contains the index
+/// where the item from the first sequence appears in the 2nd sequence or `None`
+/// if the item doesn't appear in the 2nd sequence. The 2nd tuple is the same
+/// but listing the corresponding indexes for the 2nd sequence in the first
+/// sequence.
+///
 /// # Examples
 ///
 /// ```
@@ -113,11 +128,23 @@ pub fn diff<A: PartialEq<B>, B>(a: &[A], b: &[B]) -> (Diff, Diff) {
 
 /// Returns the correspondence between two sequences with a comparison function.
 ///
+/// The return value is a pair of tuples. The first tuple contains the index
+/// where the item from the first sequence appears in the 2nd sequence or `None`
+/// if the item doesn't appear in the 2nd sequence. The 2nd tuple is the same
+/// but listing the corresponding indexes for the 2nd sequence in the first
+/// sequence.
+///
 /// # Examples
 ///
 /// ```
 /// use seqdiff;
-/// let nan_eq = |a: &f64, b: &f64| if a.is_nan() && b.is_nan() { true } else { a == b };
+/// let nan_eq = |a: &f64, b: &f64| {
+///     if a.is_nan() && b.is_nan() {
+///         true
+///     } else {
+///         a == b
+///     }
+/// };
 /// let (a2b, b2a) = seqdiff::diff_by(&[1., 2., f64::NAN], &[1., f64::NAN], nan_eq);
 /// assert_eq!(a2b, vec![Some(0), None, Some(1)]);
 /// assert_eq!(b2a, vec![Some(0), Some(2)]);
@@ -126,5 +153,30 @@ pub fn diff_by<A, B, F>(a: &[A], b: &[B], is_eq: F) -> (Diff, Diff)
 where
     F: Fn(&A, &B) -> bool,
 {
-    path_to_diff(get_shortest_edit_path_myers(a, b, is_eq))
+    path_to_diff(get_shortest_edit_path(a, b, is_eq, true).1.unwrap())
+}
+
+/// Compute similarity of two sequences.
+/// The similarity is a floating point number in [0., 100.], computed based on
+/// Levenshtein distance.
+/// This is useful, for example, fuzzy search.
+///
+/// # Examples
+///
+/// ```
+/// use seqdiff::ratio;
+/// let r = ratio(
+///     &"Hello world!".chars().collect::<Vec<_>>(),
+///     &"Holly grail!".chars().collect::<Vec<_>>(),
+/// );
+/// assert!((r - 58.333333333333337).abs() < 1e-5);
+/// ```
+#[allow(clippy::many_single_char_names)]
+pub fn ratio<A: PartialEq<B>, B>(a: &[A], b: &[B]) -> f64 {
+    let l = a.len() + b.len();
+    if l == 0 {
+        return 100.;
+    }
+    let ret = l - get_shortest_edit_path(a, b, <A as PartialEq<B>>::eq, false).0;
+    (ret * 100) as f64 / l as f64
 }
